@@ -31,17 +31,21 @@ program sound_2d
   integer, allocatable, dimension(:) :: o_flag  ! 
   integer, allocatable, dimension(:) :: c_flag  ! 
 !-- tmp
-  integer :: nttmp, i, j, k
+  integer :: nttmp, i, j, k, i_undef
 !-- parameter
   integer, parameter :: col_num=6
+  integer, parameter :: skip_num=2
 
 !-- real
   real :: z_bot       ! βü
   real :: z_top       ! ξü
   real :: dz          ! ľֳ [m]
+  real :: dvy         ! vy δֳ
   real :: smax, smin, cmax, cmin  ! 顼, ξ
   real, allocatable, dimension(:) :: time  ! 
   real, allocatable, dimension(:) :: z     ! ٺɸ
+  real, allocatable, dimension(:) :: vx     ! viewport 
+  real, allocatable, dimension(:,:) :: vy     ! viewport 
   real, allocatable, dimension(:,:) :: val_f        ! ׻
   real, allocatable, dimension(:,:) :: val_inter    ! ׻ѿ
   real, allocatable, dimension(:,:) :: tline        ! ׻ѿ
@@ -50,7 +54,7 @@ program sound_2d
   real :: undef
   real :: conv_undef
 !-- tmp
-  real, dimension(2) :: pt_tmp, ept_tmp, sept_tmp, s_int, c_int
+  real, dimension(2) :: pt_tmp, ept_tmp, sept_tmp, rh_tmp, s_int, c_int
 
 !-- character
   character(80) :: flist         ! եꥹ̾
@@ -66,6 +70,7 @@ program sound_2d
 
 !-- logical
   logical :: vec_val       ! ѿ̾
+  logical :: fflag
 
 !-- type
   type(dtime) :: start_day, end_day
@@ -78,6 +83,8 @@ program sound_2d
   &                vec_val, cnum, snum, undef
   read(5,nml=input)
 
+  fflag=.false.
+
   nz=int((z_top-z_bot)/dz)+1
   nt=line_number_counter( trim(flist) )
   s_int=(/smin, smax/)
@@ -87,8 +94,8 @@ program sound_2d
   allocate(time(nt))
   allocate(z(nz))
   allocate(inter_z(nz))
-  allocate(val_inter(col_num+3,nz))
-  allocate(val_draw(col_num+3,nt,nz))
+  allocate(val_inter(col_num+4,nz))
+  allocate(val_draw(col_num+4,nt,nz))
   allocate(year_v(nt))
   allocate(month_v(nt))
   allocate(day_v(nt))
@@ -142,7 +149,8 @@ program sound_2d
         nttmp=line_number_counter( trim(fname(1,i)) ) -2
         allocate(val_w(col_num,nttmp))
         allocate(val_f(col_num,nttmp))
-        call read_file_text( trim(fname(1,i)), col_num, nttmp, val_w, skip=2 )
+        call read_file_text( trim(fname(1,i)), col_num, nttmp, val_w,  &
+  &                          skip=skip_num )
 
      !-- convert type from character to float
         do k=1,nttmp
@@ -181,6 +189,7 @@ program sound_2d
               val_inter(col_num+1,k)=undef
               val_inter(col_num+2,k)=undef
               val_inter(col_num+3,k)=undef
+              val_inter(col_num+4,k)=undef
            else
               pt_tmp(1)=theta_dry( val_f(3,inter_z(k)), val_f(2,inter_z(k)) )
               ept_tmp(1)=thetae_Bolton( val_f(3,inter_z(k)),  &
@@ -188,12 +197,18 @@ program sound_2d
   &                                     val_f(2,inter_z(k)) )
               sept_tmp(1)=thetaes_Bolton( val_f(3,inter_z(k)),  &
   &                                       val_f(2,inter_z(k)) )
+              rh_tmp(1)=qvTP_2_RH( val_f(4,inter_z(k)),  &
+  &                                val_f(3,inter_z(k)),  &
+  &                                val_f(2,inter_z(k)) )
               pt_tmp(2)=theta_dry( val_f(3,inter_z(k)), val_f(2,inter_z(k)) )
               ept_tmp(2)=thetae_Bolton( val_f(3,inter_z(k)+1),  &
   &                                     val_f(4,inter_z(k)+1),  &
   &                                     val_f(2,inter_z(k)+1) )
               sept_tmp(2)=thetaes_Bolton( val_f(3,inter_z(k)+1),  &
   &                                       val_f(2,inter_z(k)+1) )
+              rh_tmp(2)=qvTP_2_RH( val_f(4,inter_z(k)+1),  &
+  &                                val_f(3,inter_z(k)+1),  &
+  &                                val_f(2,inter_z(k)+1) )
 
               call interpolation_1d( val_f(1,inter_z(k):inter_z(k)+1),  &
   &                                  pt_tmp, z(k), val_inter(col_num+1,k) )
@@ -201,6 +216,8 @@ program sound_2d
   &                                  ept_tmp, z(k), val_inter(col_num+2,k) )
               call interpolation_1d( val_f(1,inter_z(k):inter_z(k)+1),  &
   &                                  sept_tmp, z(k), val_inter(col_num+3,k) )
+              call interpolation_1d( val_f(1,inter_z(k):inter_z(k)+1),  &
+  &                                  rh_tmp, z(k), val_inter(col_num+4,k) )
 
            end if
         end do
@@ -212,7 +229,7 @@ program sound_2d
         val_inter=undef
      end if
 
-     do j=1,col_num+3
+     do j=1,col_num+4
         do k=1,nz
            val_draw(j,i,k)=val_inter(j,k)
         end do
@@ -226,7 +243,7 @@ program sound_2d
   select case (cont_val)
   case ('temp')
      cont_flag=3
-  case ('rh')
+  case ('qv')
      cont_flag=4
   case ('pt')
      cont_flag=col_num+1
@@ -234,6 +251,8 @@ program sound_2d
      cont_flag=col_num+2
   case ('sept')
      cont_flag=col_num+3
+  case ('rh')
+     cont_flag=col_num+4
   case ('east')
      cont_flag=5
   case ('north')
@@ -243,7 +262,7 @@ program sound_2d
   select case (shade_val)
   case ('temp')
      shade_flag=3
-  case ('rh')
+  case ('qv')
      shade_flag=4
   case ('pt')
      shade_flag=col_num+1
@@ -251,6 +270,8 @@ program sound_2d
      shade_flag=col_num+2
   case ('sept')
      shade_flag=col_num+3
+  case ('rh')
+     shade_flag=col_num+4
   case ('east')
      shade_flag=5
   case ('north')
@@ -260,7 +281,7 @@ program sound_2d
 !-- DCL part
 !-- ޤ, -
   call color_setting( snum, s_int, col_min=15, col_max=85,  &
-  &                   min_tab=10999, max_tab=999999 )
+  &                   min_tab=10999, max_tab=99999 )
 
   call DclOpenGraphics(IWS)
 
@@ -271,16 +292,29 @@ program sound_2d
   &       (/'            ', 'altitude (m)'/), stime, days, (/'(f6.1)', '(f6.1)'/),  &
   &       c_num=(/cnum, snum/), trigleg='a' )
   else
-     call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
-  &       time, z, val_draw(cont_flag,:,:), val_draw(shade_flag,:,:),  &
-  &       val_draw(5,:,:), val_draw(6,:,:), (/20,20/),  &
-  &       c_int, s_int,  &
-  &       (/'            ', 'altitude (m)'/), stime, days,  &
-  &       (/'(f6.1)', '(f6.1)'/),  &
-  &       c_num=(/cnum, snum/), trigleg='a',  &
-  &       unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
-  &       unit_fact=(/5.0e-2, 5.0e-2/),  &
-  &       unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
+     if(size(time)<20)then
+        call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
+  &          time, z, val_draw(cont_flag,:,:), val_draw(shade_flag,:,:),  &
+  &          val_draw(5,:,:), val_draw(6,:,:), (/nt,20/),  &
+  &          c_int, s_int,  &
+  &          (/'            ', 'altitude (m)'/), stime, days,  &
+  &          (/'(f6.1)', '(f6.1)'/),  &
+  &          c_num=(/cnum, snum/), trigleg='a',  &
+  &          unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
+  &          unit_fact=(/5.0e-2, 5.0e-2/),  &
+  &          unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
+     else
+        call Dcl_2D_Cont_Shade_vec_Calendar( trim(title_txt),  &
+  &          time, z, val_draw(cont_flag,:,:), val_draw(shade_flag,:,:),  &
+  &          val_draw(5,:,:), val_draw(6,:,:), (/20,20/),  &
+  &          c_int, s_int,  &
+  &          (/'            ', 'altitude (m)'/), stime, days,  &
+  &          (/'(f6.1)', '(f6.1)'/),  &
+  &          c_num=(/cnum, snum/), trigleg='a',  &
+  &          unitv=.true., vfact=(/2.0e-3, 2.0e-3/),  &
+  &          unit_fact=(/5.0e-2, 5.0e-2/),  &
+  &          unit_fact_sign=.false., unit_title=(/'U = 25m/s', ''/) )
+     end if
   end if
 
 !-- ʹ, ήѥ᡼ѿλǡ񤭽Ф
@@ -330,7 +364,7 @@ program sound_2d
      end do
 
      do i=1,nt-2
-        tmp_c=trim(fname(i_flag(1),i+2))
+        tmp_c=trim( adjustl( fname(i_flag(1),i+2) ) )
         year_v(i)=c2i_convert( trim(tmp_c(1:4)) )
         month_v(i)=c2i_convert( trim(tmp_c(5:6)) )
         day_v(i)=c2i_convert( trim(tmp_c(7:8)) )
@@ -357,18 +391,50 @@ program sound_2d
 
      if(i_counter>0)then
 
+     !-- vx  i_counter ʬ.
+        allocate(vx(2))
+        allocate(vy(i_counter,2))
+        vx=(/0.2, 0.95/)
+        dvy=(0.8-0.1*real(i_counter-1))/real(i_counter)
+        vy(1,1)=0.1
+        vy(1,2)=vy(1,1)+dvy
+        if(i_counter>1)then
+           do i=2,i_counter
+              vy(i,1)=vy(i-1,2)+0.1
+              vy(i,2)=vy(i,1)+dvy
+           end do
+        end if
+
         do i=1,nt-2
            do j=1,i_counter
-              tmp_c=trim(fname(o_flag(j),i+2))
+              tmp_c=trim( adjustl( fname(o_flag(j),i+2) ) )
               tline(i,j)=time(i)
               yline(i,j)=c2r_convert( trim(tmp_c))
            end do
         end do
       
         do j=1,i_counter
-           call Dcl_PL_Calendar( 'l', '', tline(:,j:j), yline(:,j:j),  &
-  &                              tline(:,j:j), yline(:,j:j),  &
-  &                              (/'', trim(fname(o_flag(j),1))/), stime, days )
+           i_undef=1
+           do i=1,nt-2
+              if(yline(i,j)/=undef)then
+                 exit
+              else
+                 i_undef=i_undef+1
+              end if
+           end do
+
+           if(j>1)then
+              fflag=.true.
+           end if
+           tmp_c=trim(fname(o_flag(j),1))//  &
+  &             '('//trim(fname(o_flag(j),2))//')'
+
+           call Dcl_PL_Calendar( 'l', '',   &
+  &             tline(i_undef:nt-2,j:j), yline(i_undef:nt-2,j:j),  &
+  &             tline(i_undef:nt-2,j:j), yline(i_undef:nt-2,j:j),  &
+  &             (/trim(tmp_c), trim(tmp_c)/),  &
+  &             stime, days, viewx_int=vx, viewy_int=vy(j,:),  &
+  &             no_frame=fflag )
         end do
 
      end if
@@ -388,10 +454,18 @@ program sound_2d
            tmp_c=trim(tmp_c)//', '//trim(fname(c_flag(j),1))
         end do
       
+        i_undef=1
+        do i=1,nt-2
+           if(yline(i,j)/=undef)then
+              exit
+           else
+              i_undef=i_undef+1
+           end if
+        end do
         call Dcl_PL_Calendar( 'l', '',  &
-  &                           tline(:,1:c_flag(j)), yline(:,1:c_flag(j)),  &
-  &                           tline(:,1:c_flag(j)), yline(:,1:c_flag(j)),  &
-  &                           (/'', trim(tmp_c)/), stime, days )
+  &          tline(i_undef:nt-2,1:c_flag(j)), yline(i_undef:nt-2,1:c_flag(j)),  &
+  &          tline(i_undef:nt-2,1:c_flag(j)), yline(i_undef:nt-2,1:c_flag(j)),  &
+  &          (/'', trim(tmp_c)/), stime, days )
 
      end if
 
