@@ -20,7 +20,7 @@ program sound_conv
   real :: conv_undef
   real :: limit_height, factor, ew_tmp, ww_tmp, snd_height, conv_height, dz_conv
   real, allocatable, dimension(:) :: height, pres, temp, vapor, tmp, ew, ww
-  logical :: limit_flag, conv_inter
+  logical :: limit_flag, conv_inter, continu_flag, write_flag
 
   i_undef=len_trim(undef)
 
@@ -28,7 +28,8 @@ program sound_conv
 
 !  call read_namelist()
   namelist /input /fname, undef, sign_flag, skip_num, unity, limit_height,  &
-  &                snd_height, output_name, dz_conv, conv_inter, conv_undef
+  &                snd_height, output_name, dz_conv, conv_inter, conv_undef,  &
+  &                continu_flag
   read(5,nml=input)
 
 !-- list file read
@@ -255,31 +256,42 @@ program sound_conv
      end if
 
      conv_height=limit_height
+     write_flag=.true.
 
      do j=neg_i,nz-1
         if(height(j)>limit_height)then
            if(height(j)>0.0.and.pres(j)>0.0)then
-              if(conv_inter.eqv..false.)then
-                 write(11+id,'(1000f)') height(j), pres(j), temp(j),  &
-  &                                     vapor(j), ew(j), ww(j)
-              else
-                 if(conv_height<height(j))then
+              if(write_flag.eqv..true.)then
+                 if(conv_inter.eqv..false.)then
                     write(11+id,'(1000f)') height(j), pres(j), temp(j),  &
   &                                        vapor(j), ew(j), ww(j)
-                    conv_height=conv_height+dz_conv
+                 else
+                    if(conv_height<height(j))then
+                       write(11+id,'(1000f)') height(j), pres(j), temp(j),  &
+  &                                           vapor(j), ew(j), ww(j)
+                       conv_height=conv_height+dz_conv
+                    end if
                  end if
               end if
+
               if(height(j+1)<=height(j))then
-                 exit
+                 if(continu_flag.eqv..false.)then
+                    write_flag=.true.
+                    exit
+                 else
+                    write_flag=.false.
+                 end if
+              else
+                 write_flag=.true.
               end if
            end if
         end if
-        if(j==nz-1)then
-           if(height(j+1)>0.0.and.pres(j+1)>0.0)then
-              write(11+id,'(1000f)') height(j+1), pres(j+1), temp(j+1),  &
-  &                                  vapor(j+1), ew(j+1), ww(j+1)
-           end if
-        end if
+!        if(j==nz-1)then
+!           if(height(j+1)>0.0.and.pres(j+1)>0.0)then
+!              write(11+id,'(1000f)') height(j+1), pres(j+1), temp(j+1),  &
+!  &                                  vapor(j+1), ew(j+1), ww(j+1)
+!           end if
+!        end if
      end do
      close(unit=11+id,status='keep')
 
