@@ -483,6 +483,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
   end if
 
   if(present(undeff))then
+
 !$omp parallel default(shared)
 !$omp do private(i)
 ! x ˿ľ̾Ǥ x α٥٥ȥ 3 Ƿ׻.
@@ -490,10 +491,11 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
         call curl( y, z, v(i,:,:), w(i,:,:), zeta(i,:,:), undeff,  &
   &                hx=scaley(i,:,:), hy=scalez(i,:,:) )
      end do
-
 !$omp end do
 !$omp end parallel
+
   else
+
 !$omp parallel default(shared)
 !$omp do private(i)
 ! x ˿ľ̾Ǥ x α٥٥ȥ 3 Ƿ׻.
@@ -509,6 +511,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
 ! y ˿ľ̾Ǥ y α٥٥ȥ 3 Ƿ׻.
 
   if(present(undeff))then
+
 !$omp parallel default(shared)
 !$omp do private(j)
      do j=1,ny
@@ -519,6 +522,7 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
 !$omp end parallel
 
   else
+
 !$omp parallel default(shared)
 !$omp do private(j)
      do j=1,ny
@@ -533,25 +537,27 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
 ! z ˿ľ̾Ǥ z α٥٥ȥ 3 Ƿ׻.
 
   if(present(undeff))then
+
 !$omp parallel default(shared)
 !$omp do private(k)
      do k=1,nz
         call curl( x, y, u(:,:,k), v(:,:,k), xi(:,:,k), undeff,  &
   &                hx=scalex(:,:,k), hy=scaley(:,:,k) )
      end do
-
 !$omp end do
 !$omp end parallel
+
   else
+
 !$omp parallel default(shared)
 !$omp do private(k)
      do k=1,nz
         call curl( x, y, u(:,:,k), v(:,:,k), xi(:,:,k),  &
   &                hx=scalex(:,:,k), hy=scaley(:,:,k) )
      end do
-
 !$omp end do
 !$omp end parallel
+
   end if
 
 end subroutine
@@ -974,9 +980,7 @@ subroutine zast_2_vel_3d( x, y, zast, u, v, w, uh, vh, wh, undef )
   real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: vh  ! ǥȷϤˤ [m/s]
   real, dimension(size(zast,1),size(zast,2),size(zast,3)), intent(inout) :: wh  ! ǥȷϤˤľ [m/s]
   real, intent(in), optional :: undef  ! »
-  integer :: i, j, k, nx, ny, nz
-  real, dimension(size(zast,1),size(zast,2),size(zast,3)) :: dx, dy
-  real :: cost, sint
+  integer :: k, nx, ny, nz
 
   nx=size(zast,1)
   ny=size(zast,2)
