@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module Statistics  ! ײϴطΥ롼
 
   private :: summ
@@ -601,7 +605,7 @@ subroutine nearest_search_3d( x, y, z, pointx, pointy, pointz, i, j, k )
 end subroutine nearest_search_3d
 
 
-subroutine interpo_search_1d( x, point, i, undeff )
+subroutine interpo_search_1d( x, point, i, undeff, stdopt )
   ! ǿ뤴Ȥͤ礭ʤˤΤʤ,
   ! point ֹϤ.
   implicit none
@@ -609,27 +613,38 @@ subroutine interpo_search_1d( x, point, i, undeff )
   real, intent(in) :: point  ! 
   integer, intent(inout) :: i  ! point ͤۤʤֹͤ
   integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
+                                           ! default Ǥ .false. (ɽ)
   integer :: nx, j
   integer :: just
+  logical :: stderr
 
   nx=size(x)
   if(present(undeff))then
-     just=int(undeff)
+     just=undeff
   else
-     just=0
+     just=-2147483648
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
   end if
 
   do j=1,nx
      if(x(1)>point)then
-        write(*,*) "****** WARNING ******"
-        write(*,*) "searching point was not found."
-        write(*,*) "Abort. Exit.!!!"
+        if(stderr.eqv..false.)then
+           write(*,*) "****** WARNING ******"
+           write(*,*) "searching point was not found."
+           write(*,*) "Abort. Exit.!!!"
+        end if
         i=just
         exit
      end if
 
      if(present(undeff))then
-        if(x(j)/=undeff)then
+        if(x(j)/=real(undeff))then
            if(x(j)<=point)then
               i=j
            else
@@ -648,7 +663,7 @@ subroutine interpo_search_1d( x, point, i, undeff )
 end subroutine interpo_search_1d
 
 
-subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff )
+subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff, stdopt )
   ! ǿ뤴Ȥͤ礭ʤˤΤʤ,
   ! point ֹϤ.
   implicit none
@@ -659,21 +674,30 @@ subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff )
   integer, intent(inout) :: i  ! pointx ͤۤʤֹͤ
   integer, intent(inout) :: j  ! pointy ͤۤʤֹͤ
   integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: just
+  logical :: stderr
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
 
   if(present(undeff))then
-     just=int(undeff)
-     call interpo_search_1d( x, pointx, i, just )
-     call interpo_search_1d( y, pointy, j, just )
+     just=undeff
+     call interpo_search_1d( x, pointx, i, just, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, just, stdopt=stderr )
   else
-     call interpo_search_1d( x, pointx, i )
-     call interpo_search_1d( y, pointy, j )
+     call interpo_search_1d( x, pointx, i, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, stdopt=stderr )
   end if
 
 end subroutine interpo_search_2d
 
 
-subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
+subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k,  &
+  &                           undeff, stdopt )
   ! ǿ뤴Ȥͤ礭ʤˤΤʤ,
   ! point ֹϤ.
   implicit none
@@ -687,17 +711,25 @@ subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
   integer, intent(inout) :: j  ! pointy ͤۤʤֹͤ
   integer, intent(inout) :: k  ! pointz ͤۤʤֹͤ
   integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: just
+  logical :: stderr
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
+  end if
 
   if(present(undeff))then
      just=int(undeff)
-     call interpo_search_1d( x, pointx, i, just )
-     call interpo_search_1d( y, pointy, j, just )
-     call interpo_search_1d( z, pointz, k, just )
+     call interpo_search_1d( x, pointx, i, just, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, just, stdopt=stderr )
+     call interpo_search_1d( z, pointz, k, just, stdopt=stderr )
   else
-     call interpo_search_1d( x, pointx, i )
-     call interpo_search_1d( y, pointy, j )
-     call interpo_search_1d( z, pointz, k )
+     call interpo_search_1d( x, pointx, i, stdopt=stderr )
+     call interpo_search_1d( y, pointy, j, stdopt=stderr )
+     call interpo_search_1d( z, pointz, k, stdopt=stderr )
   end if
 
 end subroutine interpo_search_3d
@@ -820,7 +852,7 @@ end subroutine interpolation_3d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_1d( x, r, u, v, undef )
+subroutine auto_interpolation_1d( x, r, u, v, undef, stdopt )
   ! ɸ x Ƥǡ u 
   ! ɸ r ǡ v ˼ưޤ.
   implicit none
@@ -828,9 +860,11 @@ subroutine auto_interpolation_1d( x, r, u, v, undef )
   real, intent(in) :: r(:)  ! ޺ɸ
   real, intent(in) :: u(size(x))  ! ǡ
   real, intent(inout) :: v(size(r))  ! ޤǡ
-  real, intent(in), optional :: undef  ! ̤
+  integer, intent(in), optional :: undef  ! ̤
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, nx, nr, ir
-  real :: defun
+  integer :: defun
+  logical :: stderr
 
   nx=size(x)
   nr=size(r)
@@ -838,11 +872,17 @@ subroutine auto_interpolation_1d( x, r, u, v, undef )
   if(present(undef))then
      defun=undef
   else
-     defun=0.0
+     defun=-2147483648
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
   end if
 
   do i=1, nr
-     call interpo_search_1d( x, r(i), ir, int(defun) )
+     call interpo_search_1d( x, r(i), ir, int(defun), stdopt=stderr )
 
      if(ir/=int(defun).and.ir<nx)then
         call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
@@ -856,7 +896,7 @@ end subroutine auto_interpolation_1d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
+subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, stdopt )
   ! ɸ x, y Ƥǡ u 
   ! ɸ r, q ǡ v ˼ưޤ.
   implicit none
@@ -866,9 +906,11 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
   real, intent(in) :: q(:)  ! ޺ɸ 2
   real, intent(in) :: u(size(x),size(y))  ! ǡ
   real, intent(inout) :: v(size(r),size(q))  ! ޤǡ
-  real, intent(in), optional :: undef  ! ̤
+  integer, intent(in), optional :: undef  ! ̤
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, nx, ny, nr, nq, ir, iq
-  real :: defun
+  integer :: defun
+  logical :: stderr
 
   nx=size(x)
   ny=size(y)
@@ -878,12 +920,19 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef )
   if(present(undef))then
      defun=undef
   else
-     defun=0.0
+     defun=-2147483648
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
   end if
 
   do j=1, nq
      do i=1, nr
-        call interpo_search_2d( x, y, r(i), q(j), ir, iq, int(defun) )
+        call interpo_search_2d( x, y, r(i), q(j), ir, iq, int(defun),  &
+  &                             stdopt=stderr )
 
         if(ir/=int(defun).and.iq/=int(defun).and.ir<nx.and.iq<ny)then
            call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
@@ -899,7 +948,7 @@ end subroutine auto_interpolation_2d
 !------------------------------------
 !------------------------------------
 
-subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
+subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, stdopt )
   ! ɸ x, y, z Ƥǡ u 
   ! ɸ r, q, p ǡ v ˼ưޤ.
   implicit none
@@ -911,9 +960,11 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
   real, intent(in) :: p(:)  ! ޺ɸ 3
   real, intent(in) :: u(size(x),size(y),size(z))  ! ǡ
   real, intent(inout) :: v(size(r),size(q),size(p))  ! ޤǡ
-  real, intent(in), optional :: undef  ! ̤
+  integer, intent(in), optional :: undef  ! ̤
+  logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, k, nx, ny, nz, nr, nq, np, ir, iq, ip
-  real :: defun
+  integer :: defun
+  logical :: stderr
 
   nx=size(x)
   ny=size(y)
@@ -925,14 +976,20 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef )
   if(present(undef))then
      defun=undef
   else
-     defun=0.0
+     defun=-2147483648
+  end if
+
+  if(present(stdopt))then
+     stderr=stdopt
+  else
+     stderr=.false.
   end if
 
   do k=1, np
      do j=1, nq
         do i=1, nr
            call interpo_search_3d( x, y, z, r(i), q(j), p(k),  &
-  &                                ir, iq, ip, int(defun) )
+  &                                ir, iq, ip, int(defun), stdopt=stderr )
 
            if(ir/=int(defun).and.iq/=int(defun).and.ip/=int(defun).and.  &
   &           ir<nx.and.iq<ny.and.ip<nz)then
@@ -969,12 +1026,14 @@ subroutine LSM_1d( x, y, slope, intercept, undef )  ! Ǿˡˤ뷹ҷ׻ (1 ǡ)
   c=0.0
   d=0.0
 
-!$omp parallel do shared(u, v, x, y) private(i)
+!$omp parallel default(shared)
+!$omp do private(i)
   do i=1,nx
      u(i)=x(i)*x(i)
      v(i)=x(i)*y(i)
   end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
 
   if(present(undef))then
      call summ(v,a,undef)
