@@ -1,12 +1,21 @@
+!-----------------------------------------------------------------------
+!     Copyright (C) 2000-2013 GFD Dennou Club. All rights reserved.
+!-----------------------------------------------------------------------
+
 module typhoon_analy  ! ټ¸ѥڥϥ⥸塼
 
+  private :: search_region_1d  ! ʿѤǽȾ¤ܺ.
+
 contains
 
-subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v )
+subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v, undef,  &
+  &                           undefg, undefgc )
   ! Ǥդʪ̤濴ʿѤ롼
   ! Υ롼®ʿѤˤѤ뤳ȤϤǤʤ.
   ! ʿѤԤݤˤ, ̤Υ롼, tangent_mean_vec λѤɬ.
   ! ¤Ȥ, |x(1)-xc|, |xc-x(nx)|, |y(1)-yc|, |yc-y(ny)| > r. 
+  ! ʳξ, r ͿΥΰ賰ϰϤˤĤƤ
+  ! undef ꤵ줿ͤ. undef ʤȤϥ.
   ! ʿѲμϰʲΤȤ.
   ! (1) nr, nt Τ٤ƤˤĤƤб x, y ɸͤ rt_2_xy Ƿ׻.
   ! (2) ޤ x,y åɤΰ interpo_search_2d Ǹ.
@@ -14,6 +23,9 @@ subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v )
   !     ,  interpolation_2d Ƿ׻.
   ! (4) nr x nt Ĥޥ顼ͤޤä, nt ʿѷ׻ mean_1d .
   ! ʾǳ nr ˤĤʿͤ.
+  ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
+  ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
+  ! βս򻲾Ȥ줿.
   use algebra
   use Derivation
   use max_min
@@ -28,38 +40,49 @@ subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v )
   real, intent(in) :: r(:)  ! ʿѲȤưκɸ(xc ͤ).
   real, intent(in) :: theta(:)  ! ʿѲȤκɸ [rad].
   real, intent(inout) :: v(size(r))  ! ʿѲ u .
-  integer :: i, j, nx, ny, nr, nt
+  real, intent(in), optional :: undef   ! ΰ賰
+  real, intent(in), optional :: undefg  ! ʻ˷»̤
+  character(3), intent(in), optional :: undefgc  ! undefg ν
+                ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
+                ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
+                ! ǥեȤ "inc".
+  integer :: i, j, nx, ny, nr, nrr, nt
   real :: work(size(r),size(theta))
   real :: point(size(r),size(theta),2)
   integer :: ip(size(r),size(theta),2)
   real :: tmpx(2), tmpy(2), tmpz(2,2), inter(2)
+  character(1) :: undefgcflag
+  logical, dimension(size(r)) :: undefgc_check
+  logical :: ucf
 
   nx=size(x)
   ny=size(y)
-  nr=size(r)
+  nrr=size(r)
   nt=size(theta)
 
-!-- ΰ򥯥ꥢƤ뤫ǧ ---
-  if(abs(x(1)-xc) < r(nr))then
-     write(*,*) "error : |x(1)-xc| >= rmax. "
-     stop
-  else
-     if(abs(x(nx)-xc) < r(nr))then
-        write(*,*) "error : |x(nx)-xc| >= rmax. "
-        stop
+  undefgc_check(:)=.true.
+
+  if(present(undefg))then
+     if(present(undefgc))then
+        undefgcflag=undefgc(1:1)
      else
-        if(abs(y(1)-yc) < r(nr))then
-           write(*,*) "error : |y(1)-yc| >= rmax. "
-           stop
-        else
-           if(abs(y(ny)-yc) < r(nr))then
-              write(*,*) "error : |y(ny)-yc| >= rmax. "
-              stop
-           end if
-        end if
+        undefgcflag="i"
      end if
   end if
 
+!-- ΰ򥯥ꥢƤ뤫ǧ ---
+
+  call search_region_1d( (/x(1),x(nx)/), (/y(1),y(ny)/), (/xc,yc/), r, nr )
+
+!--  v  undef ͤƤ.
+  do i=1,nrr
+     if(present(undef))then
+        v(i)=undef
+     else
+        v(i)=0.0
+     end if
+  end do
+
 !-- (1) ---
   do j=1,nt
      do i=1,nr
@@ -88,16 +111,46 @@ subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v )
         tmpz(2,1)=u(ip(i,j,1)+1,ip(i,j,2))
         tmpz(1,2)=u(ip(i,j,1),ip(i,j,2)+1)
         tmpz(2,2)=u(ip(i,j,1)+1,ip(i,j,2)+1)
-        inter(1)=point(i,j,1)
-        inter(2)=point(i,j,2)
-        call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
+
+        if(present(undefg))then
+           ucf=undef_checker_2d( tmpz, undefg )
+           if(ucf.eqv..false.)then
+              inter(1)=point(i,j,1)
+              inter(2)=point(i,j,2)
+              call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
+           else
+              work(i,j)=undefg
+              undefgc_check(i)=.false.
+           end if
+        else
+           inter(1)=point(i,j,1)
+           inter(2)=point(i,j,2)
+           call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
+        end if
      end do
   end do
 
 !-- (4) ---
-  do i=2,nr
-     call Mean_1d( work(i,:), v(i) )
-  end do
+  if(present(undefg))then
+     if(undefgcflag(1:1)=='i')then
+        do i=2,nr
+           call Mean_1d( work(i,:), v(i), error=undefg )
+        end do
+     else
+        do i=2,nr
+           if(undefgc_check(i).eqv..true.)then
+              call Mean_1d( work(i,:), v(i) )
+           else
+              v(i)=undefg
+           end if
+        end do
+     end if
+  else
+     do i=2,nr
+        call Mean_1d( work(i,:), v(i) )
+     end do
+  end if
+
   v(1)=work(1,1)
 
 end subroutine tangent_mean_scal
@@ -105,9 +158,12 @@ end subroutine tangent_mean_scal
 !--------------------------------------------------------------
 !--------------------------------------------------------------
 
-subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v )
+subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v, undef,  &
+  &                                undefg, undefgc )
   ! Ǥդʪ̤濴ʿѤ, ΥΥޥ꡼׻롼
   ! ¤Ȥ, |x(1)-xc|, |xc-x(nx)|, |y(1)-yc|, |yc-y(ny)| > r. 
+  ! ʳξ, r ͿΥΰ賰ϰϤˤĤƤ
+  ! undef ꤵ줿ͤ. undef ʤȤϥ.
   ! ʿѲμϰʲΤȤ.
   ! (1) nr, nt Τ٤ƤˤĤƤб x, y ɸͤ rt_2_xy Ƿ׻.
   ! (2) ޤ x,y åɤΰ interpo_search_2d Ǹ.
@@ -118,6 +174,9 @@ subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v )
   ! ܥ롼ʿ̶˺ɸåͤǤк׻Ԥ.
   ! ʿѤԤäΤ, ǥȺɸкȤˤ, 
   ! tangent_mean_anom_scal_r2c .
+  ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
+  ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
+  ! βս򻲾Ȥ줿.
   use algebra
   use Derivation
   use max_min
@@ -132,38 +191,51 @@ subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v )
   real, intent(in) :: r(:)  ! ʿѲȤưκɸ(xc ͤ).
   real, intent(in) :: theta(:)  ! ʿѲȤκɸ [rad].
   real, intent(inout) :: v(size(r),size(theta))  ! ʿѲ u ΥΥޥ꡼.
-  integer :: i, j, nx, ny, nr, nt
+  real, intent(in), optional :: undef  ! ΰ賰
+  real, intent(in), optional :: undefg  ! ʻ˷»̤
+  character(3), intent(in), optional :: undefgc  ! undefg ν
+                ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
+                ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
+                ! ǥեȤ "inc".
+  integer :: i, j, nx, ny, nr, nrr, nt
   real :: work(size(r),size(theta))
   real :: point(size(r),size(theta),2)
   integer :: ip(size(r),size(theta),2)
   real :: tmpx(2), tmpy(2), tmpz(2,2), inter(2)
+  character(1) :: undefgcflag
+  logical, dimension(size(r)) :: undefgc_check
+  logical :: ucf
 
   nx=size(x)
   ny=size(y)
-  nr=size(r)
+  nrr=size(r)
   nt=size(theta)
 
-!-- ΰ򥯥ꥢƤ뤫ǧ ---
-  if(abs(x(1)-xc) < r(nr))then
-     write(*,*) "error : |x(1)-xc| >= rmax. "
-     stop
-  else
-     if(abs(x(nx)-xc) < r(nr))then
-        write(*,*) "error : |x(nx)-xc| >= rmax. "
-        stop
+  undefgc_check(:)=.true.
+
+  if(present(undefg))then
+     if(present(undefgc))then
+        undefgcflag=undefgc(1:1)
      else
-        if(abs(y(1)-yc) < r(nr))then
-           write(*,*) "error : |y(1)-yc| >= rmax. "
-           stop
-        else
-           if(abs(y(ny)-yc) < r(nr))then
-              write(*,*) "error : |y(ny)-yc| >= rmax. "
-              stop
-           end if
-        end if
+        undefgcflag="i"
      end if
   end if
 
+!-- ΰ򥯥ꥢƤ뤫ǧ ---
+
+  call search_region_1d( (/x(1),x(nx)/), (/y(1),y(ny)/), (/xc,yc/), r, nr )
+
+!--  v  undef ͤƤ.
+  do j=1,nt
+     do i=1,nrr
+        if(present(undef))then
+           v(i,j)=undef
+        else
+           v(i,j)=0.0
+        end if
+     end do
+  end do
+
 !-- (1) ---
   do j=1,nt
      do i=1,nr
@@ -192,28 +264,64 @@ subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v )
         tmpz(2,1)=u(ip(i,j,1)+1,ip(i,j,2))
         tmpz(1,2)=u(ip(i,j,1),ip(i,j,2)+1)
         tmpz(2,2)=u(ip(i,j,1)+1,ip(i,j,2)+1)
-        inter(1)=point(i,j,1)
-        inter(2)=point(i,j,2)
-        call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
+
+        if(present(undefg))then
+           ucf=undef_checker_2d( tmpz, undefg )
+           if(ucf.eqv..false.)then
+              inter(1)=point(i,j,1)
+              inter(2)=point(i,j,2)
+              call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
+           else
+              work(i,j)=undefg
+              undefgc_check(i)=.false.
+           end if
+        else
+           inter(1)=point(i,j,1)
+           inter(2)=point(i,j,2)
+           call interpolation_2d( tmpx, tmpy, tmpz, inter, work(i,j) )
+        end if
      end do
   end do
 
 !-- (4) ---
-  do i=2,nr
-     call Anomaly_1d( work(i,:), v(i,:) )
-  end do
+  if(present(undefg))then
+     if(undefgcflag(1:1)=='i')then
+        do i=2,nr
+           call Anomaly_1d( work(i,:), v(i,:), error=undefg )
+        end do
+     else
+        do i=2,nr
+           if(undefgc_check(i).eqv..true.)then
+              call Anomaly_1d( work(i,:), v(i,:) )
+           else
+              do j=1,nt
+                 v(i,j)=undefg
+              end do
+           end if
+        end do
+     end if
+  else
+     do i=2,nr
+        call Anomaly_1d( work(i,:), v(i,:) )
+     end do
+  end if
+
   v(1,:)=work(1,1)
 
 end subroutine tangent_mean_anom_scal
 
 !--------------------------------------------------------------
 !--------------------------------------------------------------
-subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta, scal_anom, undef )
+subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
+  &                                     scal_anom, undef, undefg, undefgc )
   ! 濴Υޥ׻, ǥȺɸϤ᤹.
   ! ʿѥ롼Ѥʪ̤ʿѤ,  1 ǡݤƤ.
   ! Ʊ, ǥȷϤǤʪ̤αɸϤǤ radial ֤.
   !  radial ΰ֤ˤʿͤ 1 ǡޤǵ.
   ! ε᤿ͤ򸵤ΥǥȥǡȤǥΥޥȤ.
+  ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
+  ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
+  ! βս򻲾Ȥ줿.
   use statistics
   implicit none
   real, intent(in) :: x(:)  ! ǥȺɸϤǤ x ɸ
@@ -224,10 +332,16 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta, scal_anom,
   real, intent(in) :: r(:)  ! ʿѲȤưκɸ(xc ͤ).
   real, intent(in) :: theta(:)  ! ʿѲȤκɸ [rad].
   real, intent(inout) :: scal_anom(size(x),size(y))  ! ǥȷϤǤΥΥޥ.
-  real, optional :: undef  ! ͤĤʤȤ̤. ǥեȤǤ dcl ̤
+  real, optional :: undef  ! ͤĤʤȤ̤.
+                           ! ǥեȤǤ dcl ̤
+  real, intent(in), optional :: undefg  ! ʻ˷»̤
+  character(3), intent(in), optional :: undefgc  ! undefg ν
+                ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
+                ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
   integer :: j, k, nx, ny, nr, nt, itmpr
   real :: tmp(size(r))
   real :: tmpr, tmp_anom, undeff
+  character(3) :: undefgcflag
 
   if(present(undef))then
      undeff=undef
@@ -240,32 +354,89 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta, scal_anom,
   nr=size(r)
   nt=size(theta)
 
-  call tangent_mean_scal( x, y, xc, yc, scal, r, theta, tmp )
+  if(present(undefg))then
+     if(present(undefgc))then
+        undefgcflag=undefgc(1:3)
+     else
+        undefgcflag="inc"
+     end if
+  end if
+
+  if(present(undefg))then
+     call tangent_mean_scal( x, y, xc, yc, scal, r, theta, tmp,  &
+  &                    undef=undeff, undefg=undefg, undefgc=undefgcflag(1:3) )
+  else
+     call tangent_mean_scal( x, y, xc, yc, scal, r, theta, tmp, undef=undeff )
+  end if
 
 !-- ʿͤޤ, ͤƥΥޥ.
-  do k=1,ny
-     do j=1,nx
-        tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
-        call interpo_search_1d( r, tmpr, itmpr )
-        if(nr>itmpr)then
-           call interpolation_1d( (/r(itmpr), r(itmpr+1)/),  &
+  if(present(undefg))then
+     if(undefgcflag(1:3)=='err')then
+        do k=1,ny
+           do j=1,nx
+              tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+              call interpo_search_1d( r, tmpr, itmpr )
+              if(nr>itmpr)then
+                 if(tmp(itmpr)/=undefg.and.tmp(itmpr+1)/=undefg)then
+                    call interpolation_1d( (/r(itmpr), r(itmpr+1)/),  &
+  &                       (/tmp(itmpr), tmp(itmpr+1)/), tmpr, tmp_anom )
+                    scal_anom(j,k)=scal(j,k)-tmp_anom
+                 else
+                    scal_anom(j,k)=undefg
+                 end if
+              else
+                 scal_anom(j,k)=undeff
+              end if
+           end do
+        end do
+     else
+        do k=1,ny
+           do j=1,nx
+              tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+              call interpo_search_1d( r, tmpr, itmpr )
+              if(nr>itmpr)then
+                 if(tmp(itmpr)/=undefg.and.tmp(itmpr+1)/=undefg.and.  &
+  &                 scal(j,k)/=undefg)then
+                    call interpolation_1d( (/r(itmpr), r(itmpr+1)/),  &
+  &                       (/tmp(itmpr), tmp(itmpr+1)/), tmpr, tmp_anom )
+                    scal_anom(j,k)=scal(j,k)-tmp_anom
+                 else
+                    scal_anom(j,k)=undefg
+                 end if
+              else
+                 scal_anom(j,k)=undeff
+              end if
+           end do
+        end do
+     end if
+  else
+     do k=1,ny
+        do j=1,nx
+           tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+           call interpo_search_1d( r, tmpr, itmpr )
+           if(nr>itmpr)then
+              call interpolation_1d( (/r(itmpr), r(itmpr+1)/),  &
   &                               (/tmp(itmpr), tmp(itmpr+1)/), tmpr, tmp_anom )
-           scal_anom(j,k)=scal(j,k)-tmp_anom
-        else
-           scal_anom(j,k)=undeff
-        end if
+              scal_anom(j,k)=scal(j,k)-tmp_anom
+           else
+              scal_anom(j,k)=undeff
+           end if
+        end do
      end do
-  end do
+  end if
 
 end subroutine tangent_mean_anom_scal_Cart
 
 !--------------------------------------------------------------
 !--------------------------------------------------------------
 
-subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
+subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v,  &
+  &                          undef, undefg, undefgc )
   ! Ǥդʪ̤濴ʿѤ롼
   ! ®ʿ.
-  ! ¤Ȥ, |x(1)-xc|, |xc-x(nx)|, |y(1)-yc|, |yc-y(ny)| > r. 
+  ! ¤Ȥ, |x(1)-xc|, |xc-x(nx)|, |y(1)-yc|, |yc-y(ny)| > r.
+  ! ʳξ, r ͿΥΰ賰ϰϤˤĤƤ
+  ! undef ꤵ줿ͤ. undef ʤȤϥ.
   ! ʿѲμϰʲΤȤ.
   ! (1) nr, nt Τ٤ƤˤĤƤб x, y ɸͤ rt_2_xy Ƿ׻.
   ! (2) ޤ x,y åɤΰ interpo_search_2d Ǹ.
@@ -276,6 +447,9 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
   !     . Ʊ롼濴εΥǳä v ʬΤ.
   ! (4) nr x nt Ĥʬͤޤä, nt ʿѷ׻ mean_1d .
   ! ʾǳ nr ˤĤʿͤ.
+  ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
+  ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
+  ! βս򻲾Ȥ줿.
   use algebra
   use Derivation
   use max_min
@@ -292,7 +466,12 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
   real, intent(in) :: r(:)  ! ʿѲȤưκɸ(xc ͤ).
   real, intent(in) :: theta(:)  ! ʿѲȤκɸ [rad].
   real, intent(inout) :: v(size(r))  ! ʿѲ u .
-  integer :: i, j, nx, ny, nr, nt
+  real, intent(in), optional :: undef  ! ΰ賰
+  real, intent(in), optional :: undefg  ! ʻ˷»̤
+  character(3), intent(in), optional :: undefgc  ! undefg ν
+                ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
+                ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
+  integer :: i, j, nx, ny, nr, nrr, nt
   real :: work1(size(r),size(theta),1), work2(size(r),size(theta),1), work3(size(r),size(theta),1)
   real :: posx(size(r),size(theta),1), posy(size(r),size(theta),1), posz(size(r),size(theta),1)
   real :: vecx(size(r),size(theta),1), vecy(size(r),size(theta),1), vecz(size(r),size(theta),1)
@@ -300,33 +479,38 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
   real :: point(size(r),size(theta),2)
   integer :: ip(size(r),size(theta),2)
   real :: tmpx(2), tmpy(2), tmpz(2,2), inter(2)
+  character(1) :: undefgcflag
+  logical, dimension(size(r)) :: undefgc_check
+  logical :: ucf
 
   nx=size(x)
   ny=size(y)
-  nr=size(r)
+  nrr=size(r)
   nt=size(theta)
 
-!-- ΰ򥯥ꥢƤ뤫ǧ ---
-  if(abs(x(1)-xc) < r(nr))then
-     write(*,*) "error : |x(1)-xc| >= rmax. "
-     stop
-  else
-     if(abs(x(nx)-xc) < r(nr))then
-        write(*,*) "error : |x(nx)-xc| >= rmax. "
-        stop
+  undefgc_check(:)=.true.
+
+  if(present(undefg))then
+     if(present(undefgc))then
+        undefgcflag=undefgc(1:1)
      else
-        if(abs(y(1)-yc) < r(nr))then
-           write(*,*) "error : |y(1)-yc| >= rmax. "
-           stop
-        else
-           if(abs(y(ny)-yc) < r(nr))then
-              write(*,*) "error : |y(ny)-yc| >= rmax. "
-              stop
-           end if
-        end if
+        undefgcflag="i"
      end if
   end if
 
+!-- ΰ򥯥ꥢƤ뤫ǧ ---
+
+  call search_region_1d( (/x(1),x(nx)/), (/y(1),y(ny)/), (/xc,yc/), r, nr )
+
+!--  v  undef ͤƤ.
+  do i=1,nrr
+     if(present(undef))then
+        v(i)=undef
+     else
+        v(i)=0.0
+     end if
+  end do
+
 !-- (1) ---
   do j=1,nt
      do i=1,nr
@@ -347,26 +531,63 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
 
 !-- (3) ---
 !-- 1. ٥ȥ 2 ʬˤĤޤͤ˳Ǽ ---
-  do j=1,nt
-     do i=1,nr
-        tmpx(1)=x(ip(i,j,1))
-        tmpx(2)=x(ip(i,j,1)+1)
-        tmpy(1)=y(ip(i,j,2))
-        tmpy(2)=y(ip(i,j,2)+1)
-        tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
-        tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
-        tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
-        tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
-        inter(1)=point(i,j,1)
-        inter(2)=point(i,j,2)
-        call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
-        tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
-        tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
-        tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
-        tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
-        call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
+  if(present(undefg))then
+     do j=1,nt
+        do i=1,nr
+           tmpx(1)=x(ip(i,j,1))
+           tmpx(2)=x(ip(i,j,1)+1)
+           tmpy(1)=y(ip(i,j,2))
+           tmpy(2)=y(ip(i,j,2)+1)
+           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
+           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
+           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
+           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
+           ucf=undef_checker_2d( tmpz, undefg )
+           if(ucf.eqv..false.)then
+              inter(1)=point(i,j,1)
+              inter(2)=point(i,j,2)
+
+              call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
+
+              tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
+              tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
+              tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
+              tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
+              ucf=undef_checker_2d( tmpz, undefg )
+              if(ucf.eqv..false.)then
+                 call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
+              else
+                 work1(i,j,1)=undefg
+                 work2(i,j,1)=undefg
+              end if
+           else  ! λ undefg äƤ, work2 Ȥʤ.
+              work1(i,j,1)=undefg
+              work2(i,j,1)=undefg
+           end if
+        end do
      end do
-  end do
+  else
+     do j=1,nt
+        do i=1,nr
+           tmpx(1)=x(ip(i,j,1))
+           tmpx(2)=x(ip(i,j,1)+1)
+           tmpy(1)=y(ip(i,j,2))
+           tmpy(2)=y(ip(i,j,2)+1)
+           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
+           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
+           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
+           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
+           inter(1)=point(i,j,1)
+           inter(2)=point(i,j,2)
+           call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
+           tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
+           tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
+           tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
+           tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
+           call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
+        end do
+     end do
+  end if
 
 !-- 2. ޤȤä, Ǥ x,y ɸ(ֹ, ix,iy ˳Ǽ)
 !--    ΰ֥٥ȥȤγѤ׻
@@ -389,21 +610,63 @@ subroutine tangent_mean_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
      stop
   end select
 
+!-- , work1  undefg äƤ뤫å.
+!-- , work1  undefg äƤ, work2 ˤ undefg 
+!-- äƤΤ, work1 åФ褤.
+
+  if(present(undefg))then
+     do j=1,nt
+        do i=1,nr
+           if(work1(i,j,1)==undefg)then
+              vecz(i,j,1)=undefg
+           end if
+        end do
+     end do
+  end if
+
 !-- 3. ٥ȥγʬΤ, z ʬˤĤ (2 ʿ̥٥ȥƱΤγ)
 !--    ֥٥ȥͤǳ. -> ®ʬޤͤ.
   call abst( posx, posy, posz, abpos )
 
-  do j=1,nt
-     do i=2,nr   
-        vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+  if(present(undefg))then
+     do j=1,nt
+        do i=2,nr   
+           if(vecz(i,j,1)/=undefg)then
+              vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+           end if
+        end do
      end do
-  end do
+  else
+     do j=1,nt
+        do i=2,nr   
+           vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+        end do
+     end do
+  end if
 
 
 !-- (4) ---
-  do i=2,nr
-     call Mean_1d( vecz(i,:,1), v(i) )
-  end do
+  if(present(undefg))then
+     if(undefgcflag(1:1)=='i')then
+        do i=2,nr
+           call Mean_1d( vecz(i,:,1), v(i), error=undefg )
+        end do
+     else
+        do i=2,nr
+           ucf=undef_checker_1d( vecz(i,:,1), undefg )
+           if(ucf.eqv..false.)then
+              call Mean_1d( vecz(i,:,1), v(i) )
+           else
+              v(i)=undefg
+           end if
+        end do
+     end if
+  else
+     do i=2,nr
+        call Mean_1d( vecz(i,:,1), v(i) )
+     end do
+  end if
+
   v(1)=0.0
 
 end subroutine tangent_mean_vec
@@ -412,10 +675,13 @@ end subroutine tangent_mean_vec
 !--------------------------------------------------------------
 !--------------------------------------------------------------
 
-subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
+subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v,  &
+  &                               undef, undefg, undefgc )
   ! Ǥդʪ̤濴ʿѥΥޥ׻롼
   ! ®ʿ.
   ! ¤Ȥ, |x(1)-xc|, |xc-x(nx)|, |y(1)-yc|, |yc-y(ny)| > r. 
+  ! ʳξ, r ͿΥΰ賰ϰϤˤĤƤ
+  ! undef ꤵ줿ͤ. undef ʤȤϥ.
   ! ʿѲμϰʲΤȤ.
   ! (1) nr, nt Τ٤ƤˤĤƤб x, y ɸͤ rt_2_xy Ƿ׻.
   ! (2) ޤ x,y åɤΰ interpo_search_2d Ǹ.
@@ -426,6 +692,9 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
   !     . Ʊ롼濴εΥǳä v ʬΤ.
   ! (4) nr x nt Ĥʬͤޤä, nt ʿѷ׻ mean_1d .
   ! ʾǳ nr ˤĤʿͤ.
+  ! ʲ, Թǽ꡹ present(undefg) äƤ뤬,
+  ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
+  ! βս򻲾Ȥ줿.
   use algebra
   use Derivation
   use max_min
@@ -442,7 +711,12 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
   real, intent(in) :: r(:)  ! ʿѲȤưκɸ(xc ͤ).
   real, intent(in) :: theta(:)  ! ʿѲȤκɸ [rad].
   real, intent(inout) :: v(size(r),size(theta))  ! Υޥ u .
-  integer :: i, j, nx, ny, nr, nt
+  real, intent(in), optional :: undef  ! ΰ賰
+  real, intent(in), optional :: undefg  ! ʻ˷»̤
+  character(3), intent(in), optional :: undefgc  ! undefg ν
+                ! "inc" = γʻ򻲾ͤȤޤΤʿ˽Ʒ׻.
+                ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
+  integer :: i, j, nx, ny, nr, nrr, nt
   real :: work1(size(r),size(theta),1), work2(size(r),size(theta),1), work3(size(r),size(theta),1)
   real :: posx(size(r),size(theta),1), posy(size(r),size(theta),1), posz(size(r),size(theta),1)
   real :: vecx(size(r),size(theta),1), vecy(size(r),size(theta),1), vecz(size(r),size(theta),1)
@@ -450,33 +724,40 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
   real :: point(size(r),size(theta),2)
   integer :: ip(size(r),size(theta),2)
   real :: tmpx(2), tmpy(2), tmpz(2,2), inter(2)
+  character(1) :: undefgcflag
+  logical, dimension(size(r)) :: undefgc_check
+  logical :: ucf
 
   nx=size(x)
   ny=size(y)
-  nr=size(r)
+  nrr=size(r)
   nt=size(theta)
 
-!-- ΰ򥯥ꥢƤ뤫ǧ ---
-  if(abs(x(1)-xc) < r(nr))then
-     write(*,*) "error : |x(1)-xc| >= rmax. "
-     stop
-  else
-     if(abs(x(nx)-xc) < r(nr))then
-        write(*,*) "error : |x(nx)-xc| >= rmax. "
-        stop
+  undefgc_check(:)=.true.
+
+  if(present(undefg))then
+     if(present(undefgc))then
+        undefgcflag=undefgc(1:1)
      else
-        if(abs(y(1)-yc) < r(nr))then
-           write(*,*) "error : |y(1)-yc| >= rmax. "
-           stop
-        else
-           if(abs(y(ny)-yc) < r(nr))then
-              write(*,*) "error : |y(ny)-yc| >= rmax. "
-              stop
-           end if
-        end if
+        undefgcflag="i"
      end if
   end if
 
+!-- ΰ򥯥ꥢƤ뤫ǧ ---
+
+  call search_region_1d( (/x(1),x(nx)/), (/y(1),y(ny)/), (/xc,yc/), r, nr )
+
+!--  v  undef ͤƤ.
+  do j=1,nt
+     do i=1,nrr
+        if(present(undef))then
+           v(i,j)=undef
+        else
+           v(i,j)=0.0
+        end if
+     end do
+  end do
+
 !-- (1) ---
   do j=1,nt
      do i=1,nr
@@ -497,26 +778,63 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
 
 !-- (3) ---
 !-- 1. ٥ȥ 2 ʬˤĤޤͤ˳Ǽ ---
-  do j=1,nt
-     do i=1,nr
-        tmpx(1)=x(ip(i,j,1))
-        tmpx(2)=x(ip(i,j,1)+1)
-        tmpy(1)=y(ip(i,j,2))
-        tmpy(2)=y(ip(i,j,2)+1)
-        tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
-        tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
-        tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
-        tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
-        inter(1)=point(i,j,1)
-        inter(2)=point(i,j,2)
-        call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
-        tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
-        tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
-        tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
-        tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
-        call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
+  if(present(undefg))then
+     do j=1,nt
+        do i=1,nr
+           tmpx(1)=x(ip(i,j,1))
+           tmpx(2)=x(ip(i,j,1)+1)
+           tmpy(1)=y(ip(i,j,2))
+           tmpy(2)=y(ip(i,j,2)+1)
+           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
+           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
+           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
+           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
+           ucf=undef_checker_2d( tmpz, undefg )
+           if(ucf.eqv..false.)then
+              inter(1)=point(i,j,1)
+              inter(2)=point(i,j,2)
+
+              call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
+
+              tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
+              tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
+              tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
+              tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
+              ucf=undef_checker_2d( tmpz, undefg )
+              if(ucf.eqv..false.)then
+                 call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
+              else
+                 work1(i,j,1)=undefg
+                 work2(i,j,1)=undefg
+              end if
+           else  ! λ undefg äƤ, work2 Ȥʤ.
+              work1(i,j,1)=undefg
+              work2(i,j,1)=undefg
+           end if
+        end do
      end do
-  end do
+  else
+     do j=1,nt
+        do i=1,nr
+           tmpx(1)=x(ip(i,j,1))
+           tmpx(2)=x(ip(i,j,1)+1)
+           tmpy(1)=y(ip(i,j,2))
+           tmpy(2)=y(ip(i,j,2)+1)
+           tmpz(1,1)=u1(ip(i,j,1),ip(i,j,2))
+           tmpz(2,1)=u1(ip(i,j,1)+1,ip(i,j,2))
+           tmpz(1,2)=u1(ip(i,j,1),ip(i,j,2)+1)
+           tmpz(2,2)=u1(ip(i,j,1)+1,ip(i,j,2)+1)
+           inter(1)=point(i,j,1)
+           inter(2)=point(i,j,2)
+           call interpolation_2d( tmpx, tmpy, tmpz, inter, work1(i,j,1) )
+           tmpz(1,1)=u2(ip(i,j,1),ip(i,j,2))
+           tmpz(2,1)=u2(ip(i,j,1)+1,ip(i,j,2))
+           tmpz(1,2)=u2(ip(i,j,1),ip(i,j,2)+1)
+           tmpz(2,2)=u2(ip(i,j,1)+1,ip(i,j,2)+1)
+           call interpolation_2d( tmpx, tmpy, tmpz, inter, work2(i,j,1) )
+        end do
+     end do
+  end if
 
 !-- 2. ޤȤä, Ǥ x,y ɸ(ֹ, ix,iy ˳Ǽ)
 !--    ΰ֥٥ȥȤγѤ׻
@@ -539,21 +857,63 @@ subroutine tangent_mean_anom_vec( charc, x, y, xc, yc, u1, u2, r, theta, v )
      stop
   end select
 
+!-- , work1  undefg äƤ뤫å.
+!-- , work1  undefg äƤ, work2 ˤ undefg 
+!-- äƤΤ, work1 åФ褤.
+
+  if(present(undefg))then
+     do j=1,nt
+        do i=1,nr
+           if(work1(i,j,1)==undefg)then
+              vecz(i,j,1)=undefg
+           end if
+        end do
+     end do
+  end if
+
 !-- 3. ٥ȥγʬΤ, z ʬˤĤ (2 ʿ̥٥ȥƱΤγ)
 !--    ֥٥ȥͤǳ. -> ®ʬޤͤ.
   call abst( posx, posy, posz, abpos )
 
-  do j=1,nt
-     do i=2,nr   
-        vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+  if(present(undefg))then
+     do j=1,nt
+        do i=2,nr
+           if(vecz(i,j,1)/=undefg)then
+              vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+           end if
+        end do
      end do
-  end do
+  else
+     do j=1,nt
+        do i=2,nr
+           vecz(i,j,1)=vecz(i,j,1)/abpos(i,j)
+        end do
+     end do
+  end if
 
 
 !-- (4) ---
-  do i=2,nr
-     call Anomaly_1d( vecz(i,:,1), v(i,:) )
-  end do
+  if(present(undefg))then
+     if(undefgcflag(1:1)=='i')then
+        do i=2,nr
+           call Anomaly_1d( vecz(i,:,1), v(i,:), error=undefg )
+        end do
+     else
+        do i=2,nr
+           ucf=undef_checker_1d( vecz(i,:,1), undefg )
+           if(ucf.eqv..false.)then
+              call Anomaly_1d( vecz(i,:,1), v(i,:) )
+           else
+              v(i,:)=undefg
+           end if
+        end do
+     end if
+  else
+     do i=2,nr
+        call Anomaly_1d( vecz(i,:,1), v(i,:) )
+     end do
+  end if
+
   v(1,:)=0.0
 
 end subroutine tangent_mean_anom_vec
@@ -910,7 +1270,7 @@ subroutine tangent_mean_deform( signal, r, z, u, v, w, val, undef )
 
   if(signal(1:2)=='22')then
 !$omp parallel default(shared)
-!$omp do private(j,k)
+!$omp do schedule(dynamic) private(j,k)
      do k=1,nz
         do j=1,nr
            if(r(j)/=0.0)then
@@ -1072,8 +1432,151 @@ subroutine grad_wind_pres( r, coril, v, rho, r_ref, p_ref, pres )
 
 end subroutine
 
+!--------------------------------------------------------------
+!  ʲ, private 롼
+!--------------------------------------------------------------
 
+subroutine search_region_1d( x, y, c, r, nr )
+! ʿѲǽȾ¤׻롼
+  use Statistics
+  implicit none
+  real, intent(in) :: x(2)  ! x ξüɸ [m]
+  real, intent(in) :: y(2)  ! y ξüɸ [m]
+  real, intent(in) :: c(2)  ! 濴ΰֺɸ (x,y) [m]
+  real, intent(in) :: r(:)  ! ưΰֺɸ [m]
+  integer, intent(inout) :: nr  ! ʿѲǽȾ¤ֹ (r(nr) ǽȾ)
+  integer :: nrr, tmp_nr
+  real :: xc, yc
 
+  nrr=size(r)
+  xc=c(1)
+  yc=c(2)
+
+  nr=nrr
+  if(abs(x(1)-xc) < r(nrr))then
+     write(*,*) "typhoon_analy WARNING :"
+     write(*,*) "|x(1)-xc| >= rmax. "
+     write(*,*) "undef value is substituted out of region."
+     call interpo_search_1d( r, abs(x(1)-xc), tmp_nr )
+     nr=tmp_nr+1  ! interpo_search  abs ͤ꾮 r ֹ椬뤿.
+                  ! ʲƱͳ
+  else
+     if(abs(x(2)-xc) < r(nrr))then
+     write(*,*) "typhoon_analy WARNING :"
+        write(*,*) "|x(nx)-xc| >= rmax. "
+        write(*,*) "undef value is substituted out of region."
+        call interpo_search_1d( r, abs(x(2)-xc), tmp_nr )
+        if(tmp_nr+1<nr)then
+           nr=tmp_nr+1
+        end if
+     else
+        if(abs(y(1)-yc) < r(nrr))then
+           write(*,*) "typhoon_analy WARNING :"
+           write(*,*) "|y(1)-yc| >= rmax. "
+           write(*,*) "undef value is substituted out of region."
+           call interpo_search_1d( r, abs(y(1)-yc), tmp_nr )
+           if(tmp_nr+1<nr)then
+              nr=tmp_nr+1
+           end if
+        else
+           if(abs(y(2)-yc) < r(nrr))then
+              write(*,*) "typhoon_analy WARNING :"
+              write(*,*) "|y(ny)-yc| >= rmax. "
+              write(*,*) "undef value is substituted out of region."
+              call interpo_search_1d( r, abs(y(2)-yc), tmp_nr )
+              if(tmp_nr+1<nr)then
+                 nr=tmp_nr+1
+              end if
+           end if
+        end if
+     end if
+  end if
+
+end subroutine search_region_1d
+
+!--------------------------------------------------------------
+!--------------------------------------------------------------
+
+logical function undef_checker_1d( val, undef )
+! Ǥ val ˤĤ, ٤ƤǤˤĤ undef ȤͤäƤ뤫
+! ɤå. 1 ĤǤ undef äƤ .true. ֤.
+  implicit none
+  real, dimension(:), intent(in) :: val  ! å
+  real, intent(in) :: undef  ! åѿ
+  integer :: i, nx
+  logical :: checker
+
+  nx=size(val)
+  checker=.false.
+
+  do i=1,nx
+     if(val(i)==undef)then
+        checker=.true.
+        exit
+     end if
+  end do
+
+  undef_checker_1d=checker
+
+  return
+end function
+
+!--------------------------------------------------------------
+!--------------------------------------------------------------
+
+logical function undef_checker_2d( val, undef )
+! Ǥ val ˤĤ, ٤ƤǤˤĤ undef ȤͤäƤ뤫
+! ɤå. 1 ĤǤ undef äƤ .true. ֤.
+  implicit none
+  real, dimension(:,:), intent(in) :: val  ! å
+  real, intent(in) :: undef  ! åѿ
+  integer :: i, nx
+  logical :: checker
+
+  nx=size(val,2)
+  checker=.false.
+
+  do i=1,nx
+     checker=undef_checker_1d( val(:,i), undef )
+     if(checker.eqv..true.)then
+        exit
+     end if
+  end do
+
+  undef_checker_2d=checker
+
+  return
+end function
+
+!--------------------------------------------------------------
+!--------------------------------------------------------------
+
+logical function undef_checker_3d( val, undef )
+! Ǥ val ˤĤ, ٤ƤǤˤĤ undef ȤͤäƤ뤫
+! ɤå. 1 ĤǤ undef äƤ .true. ֤.
+  implicit none
+  real, dimension(:,:,:), intent(in) :: val  ! å
+  real, intent(in) :: undef  ! åѿ
+  integer :: i, nx
+  logical :: checker
+
+  nx=size(val,3)
+  checker=.false.
+
+  do i=1,nx
+     checker=undef_checker_2d( val(:,:,i), undef )
+     if(checker.eqv..true.)then
+        exit
+     end if
+  end do
+
+  undef_checker_3d=checker
+
+  return
+end function
+
+!--------------------------------------------------------------
+!--------------------------------------------------------------
 
 !subroutine Rangular_moment(xp,yp,x,y,u,v,mome)
 ! Ǥդޤгѱư̤׻롼
