@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module Special_Function  !-- üؿ׻⥸塼 ---
 
 interface kaijo
@@ -800,7 +804,7 @@ double precision function bessy_d( n, z )
   integer, intent(in) :: n  ! 
   double precision, intent(in) :: z  ! ѿ
   double precision :: dt, term1, term2, tmp1, tmp2, t
-  double precision,parameter :: tmin=0.0, tmax=dble(pi)  ! ʬʬΰ
+  double precision :: tmin, tmax  ! ʬʬΰ
   double precision, parameter :: thres=1.0d-6
   integer :: i
   integer, parameter :: nt=100  ! ʬʬʬ
@@ -812,6 +816,8 @@ double precision function bessy_d( n, z )
      stop
   end if
 
+  tmin=0.0d0
+  tmax=dble(pi)
   term1=0.0d0
   term2=0.0d0
   dt=1.0d-2
